==> kmuddy-0.7.1.tar.gz: kmuddy-0.7.1/kmuddy/ctriggerlist.cpp <==
/***************************************************************************
                          ctriggerlist.cpp  -  trigger list
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne oct 13 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctriggerlist.h"

#include "ccmdprocessor.h"
#include "cconnection.h"
#include "cconnprefs.h"
#include "csession.h"
#include "ctextchunk.h"
#include "ctrigger.h"
#include "kmuddy.h"

extern KMuddy *kmuddy;

#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>

//used in loop detection
#define MAXTRIGGERS 50

cTriggerList::cTriggerList (QString filename, cSession *_sess)
  : cSaveableList (_sess)
{
  prototype = new cTrigger (_sess);
  objName = "Trigger";
  config = new KConfig (filename);
  load ();
  count = 0;
  curline = 0;
  time.start ();
}

cTriggerList::~cTriggerList ()
{
  save ();
  config->sync ();
}

void cTriggerList::matchString (cTextChunk *line)
{
  if (_enabled) //only if processing is enabled
  {
    //prepare the text to match
    curline = line;
    plainText = curline->plainText();
    
    //cycle through all items in the list...
    for (reset(); *this; (*this)++)
    {
      if (!_enabled)
        break;
      //match against alias/trigger/whatever
      bool matched = cur->match (plainText);
      if (matched)
      {
        cur->performAction ();
        processCommands (cur->commandsToExec ());
      }
      //break the loop if needed
      if (!(cur->shouldContinue (matched)))
        break;
      //also break if we're in recursion
      if (recursionAlert)
        break;
    }
    reset ();
  }
  if (recursionAlert)
  {
    //inalert is here to prevent multiple errorboxes...
    static bool inalert = false;
    if (inalert)
      _enabled = false;
    else
    {
      inalert = true;
      //_enabled=false would be enough, but I also want that message to appear...
      sess->connection()->switchTriggers (false);
      processRecursion ();
      recursionAlert = false;
      //the same as above...
      //something could have changed in the meantime...
      if (sess->connection())
        sess->connection()->switchTriggers (true);
      inalert = false;
    }
  }
}

void cTriggerList::processCommands (const QStringList &commands)
{
  if (commands.isEmpty())  //do nothing if there are no commands
    return;

  //check if loop detection is enabled
  //for quick connections, loop detection is always enabled
  cConnPrefs *cp = sess->connection()->getConnPrefs();
  bool detectLoops = ((cp == 0) ? true : cp->limitTriggers());

  if (detectLoops)
  {
    //if time diff between triggers was less than 1 second, remember it...
    if (time.elapsed () < 1000)
      count++;
    else
      count = 0;
    time.restart ();
  }
  else
    count = 0;  //no loop detection -> no trigger counting
  if (count == MAXTRIGGERS)
  {
    _enabled = false;
    recursionAlert = true;  //we're in recursion!!!
  }
  else
  {
    //okay, send the commands!
    QStringList::const_iterator it;
    for (it = commands.begin(); it != commands.end(); ++it)
      sess->cmdproc()->processTriggerInput(*it);
  }
}

void cTriggerList::processRecursion ()
{
  //triggers are now disabled; they will be reenabled when this function ends
  KMessageBox::error (kmuddy, i18n ("Trigger loop has been detected. "
      "Triggers will be temporarily disabled until you close this dialog box.\n\n"
      "Note that you can disable this check in Profile / MUD Preferences."),
      i18n ("Trigger loop detected"));
}

void cTriggerList::rewriteText (int pos, int len, const QString &newtext)
{
  //perform the actual replace
  curline->replace (pos, len, newtext);
  
  //plainText has been changed - regenerate it
  plainText = curline->plainText();
}

==> kmuddy-0.8.tar.gz: kmuddy-0.8/kmuddy/ctriggerlist.cpp <==
/***************************************************************************
                          ctriggerlist.cpp  -  trigger list
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne oct 13 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctriggerlist.h"

#include "ccmdqueue.h"
#include "ccmdqueues.h"
#include "cconnection.h"
#include "cconnprefs.h"
#include "cexpresolver.h"
#include "ctextchunk.h"
#include "ctrigger.h"
#include "kmuddy.h"

extern KMuddy *kmuddy;

#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>

//used in loop detection
#define MAXTRIGGERS 50

cTriggerList::cTriggerList (QString filename, int sess)
  : cSaveableList ("triggerlist", sess)
{
  prototype = new cTrigger (sess);
  objName = "Trigger";
  config = new KConfig (filename);
  load ();
  count = 0;
  curline = 0;
  time.start ();
  resolver = new cExpResolver (sess);
}

cTriggerList::~cTriggerList ()
{
  delete resolver;
  save ();
  config->sync ();
}

void cTriggerList::matchString (cTextChunk *line)
{
  cConnection *connection = dynamic_cast<cConnection *>(object ("connection", sess()));
  if (!connection) return;
  if (_enabled) //only if processing is enabled
  {
    //prepare the text to match
    curline = line;
    plainText = curline->plainText();
    
    //cycle through all items in the list...
    for (reset(); *this; (*this)++)
    {
      if (!_enabled)
        break;
      unsigned int mpos = 0;  // reset the index where matching will start
      bool everMatched = false;
      //match against trigger
      while (true) {
        bool matched, condmatched = true;
        cur->setMatchingPos (mpos);  // set position of matching
        matched = cur->match (plainText);
        if (matched)
        {
          condmatched = cur->testCondition (sess(), resolver);
          if (condmatched)
          {
            cur->performAction ();
            processCommands (cur->commandsToExec ());
            everMatched = true;
          }
        }
        if ((!matched) || (!cur->global()))
          break;  // not matched or not global - no further repeats

        // no global matching for rewrite triggers - too many problems with that :)
        if (((cTrigger *) cur)->isRewriteTrigger ())
          break;
        // global - update matching position, or terminate if no more matching should occur
        // if last length is 0, we must advance by 1 to avoid an endless loop ...
        // TODO rewrite trigs fail on this one
        int shift = (cur->getLastLength() == 0) ? 1 : cur->getLastLength();
        mpos = cur->getLastPos() + shift;
        if (mpos >= plainText.length())
          break;
      }
      //break the loop if needed
      if (!(cur->shouldContinue (everMatched)))
        break;
      //also break if we're in recursion
      if (recursionAlert)
        break;
    }
    reset ();
  }
  if (recursionAlert)
  {
    //inalert is here to prevent multiple errorboxes...
    static bool inalert = false;
    if (inalert)
      _enabled = false;
    else
    {
      inalert = true;
      //_enabled=false would be enough, but I also want that message to appear...
      connection->switchTriggers (false);
      processRecursion ();
      recursionAlert = false;
      //the same as above...
      //something could have changed in the meantime...
      connection->switchTriggers (true);
      inalert = false;
    }
  }
}

void cTriggerList::processCommands (const QStringList &commands)
{
  if (commands.isEmpty())  //do nothing if there are no commands
    return;

  //check if loop detection is enabled
  //for quick connections, loop detection is always enabled
  cConnPrefs *cp = dynamic_cast<cConnPrefs *>(object ("connprefs", sess()));
  bool detectLoops = ((cp == 0) ? true : cp->limitTriggers());

  if (detectLoops)
  {
    //if time diff between triggers was less than 1 second, remember it...
    if (time.elapsed () < 1000)
      count++;
    else
      count = 0;
    time.restart ();
  }
  else
    count = 0;  //no loop detection -> no trigger counting
  if (count == MAXTRIGGERS)
  {
    _enabled = false;
    recursionAlert = true;  //we're in recursion!!!
  }
  else
  {
    //okay, send the commands!
    cCmdQueues *queues = (cCmdQueues *) object ("cmdqueues");
    if (!queues) return;
    cCmdQueue *queue = new cCmdQueue (sess());
    QStringList::const_iterator it;
    for (it = commands.begin(); it != commands.end(); ++it)
      queue->addCommand (*it);
    queues->addQueue (queue);
  }
}

void cTriggerList::processRecursion ()
{
  //triggers are now disabled; they will be reenabled when this function ends
  KMessageBox::error (kmuddy, i18n ("Trigger loop has been detected. "
      "Triggers will be temporarily disabled until you close this dialog box.\n\n"
      "Note that you can disable this check in Profile / MUD Preferences."),
      i18n ("Trigger loop detected"));
}

void cTriggerList::rewriteText (int pos, int len, const QString &newtext)
{
  //perform the actual replace
  curline->replace (pos, len, newtext);
  
  //plainText has been changed - regenerate it
  plainText = curline->plainText();
}
